from setuptools import setup, find_packages

setup(
    name='word2vec-SISG',
    version='1.0.0',
    description='<Billion-scale Recommendation with Heterogeneous Side Information at Taobao> PyTorch implementation',
    packages=find_packages(include=['word2vec']),
    install_requires=[],
)

